# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dataclay/communication/grpc/generated/logicmodule/logicmodule.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from dataclay.communication.grpc.messages.logicmodule import logicmodule_messages_pb2 as dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2
from dataclay.communication.grpc.messages.common import common_messages_pb2 as dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='dataclay/communication/grpc/generated/logicmodule/logicmodule.proto',
  package='dataclay.communication.grpc.logicmodule',
  syntax='proto3',
  serialized_options=_b('\n1dataclay.communication.grpc.generated.logicmoduleB\026LogicModuleGrpcService'),
  serialized_pb=_b('\nCdataclay/communication/grpc/generated/logicmodule/logicmodule.proto\x12\'dataclay.communication.grpc.logicmodule\x1aKdataclay/communication/grpc/messages/logicmodule/logicmodule_messages.proto\x1a\x41\x64\x61taclay/communication/grpc/messages/common/common_messages.proto2\x8c\x89\x01\n\x0bLogicModule\x12\x85\x01\n\x0e\x61utoregisterSL\x12>.dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x93\x01\n\x0e\x61utoregisterEE\x12>.dataclay.communication.grpc.logicmodule.AutoRegisterEERequest\x1a?.dataclay.communication.grpc.logicmodule.AutoRegisterEEResponse\"\x00\x12\x9b\x01\n\x19unregisterStorageLocation\x12I.dataclay.communication.grpc.logicmodule.UnregisterStorageLocationRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xa5\x01\n\x1eunregisterExecutionEnvironment\x12N.dataclay.communication.grpc.logicmodule.UnregisterExecutionEnvironmentRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12s\n\ncheckAlive\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xa4\x01\n\x17performSetOfNewAccounts\x12\x42.dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest\x1a\x43.dataclay.communication.grpc.logicmodule.PerformSetAccountsResponse\"\x00\x12\xa7\x01\n\x16performSetOfOperations\x12\x44.dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest\x1a\x45.dataclay.communication.grpc.logicmodule.PerformSetOperationsResponse\"\x00\x12\x85\x01\n\x0epublishAddress\x12>.dataclay.communication.grpc.logicmodule.PublishAddressRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x95\x01\n\x11newAccountNoAdmin\x12\x41.dataclay.communication.grpc.logicmodule.NewAccountNoAdminRequest\x1a;.dataclay.communication.grpc.logicmodule.NewAccountResponse\"\x00\x12\x87\x01\n\nnewAccount\x12:.dataclay.communication.grpc.logicmodule.NewAccountRequest\x1a;.dataclay.communication.grpc.logicmodule.NewAccountResponse\"\x00\x12\x8d\x01\n\x0cgetAccountID\x12<.dataclay.communication.grpc.logicmodule.GetAccountIDRequest\x1a=.dataclay.communication.grpc.logicmodule.GetAccountIDResponse\"\x00\x12\x93\x01\n\x0egetAccountList\x12>.dataclay.communication.grpc.logicmodule.GetAccountListRequest\x1a?.dataclay.communication.grpc.logicmodule.GetAccountListResponse\"\x00\x12\x87\x01\n\nnewSession\x12:.dataclay.communication.grpc.logicmodule.NewSessionRequest\x1a;.dataclay.communication.grpc.logicmodule.NewSessionResponse\"\x00\x12\xa8\x01\n\x15getInfoOfSessionForDS\x12\x45.dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSRequest\x1a\x46.dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse\"\x00\x12\x8d\x01\n\x0cnewNamespace\x12<.dataclay.communication.grpc.logicmodule.NewNamespaceRequest\x1a=.dataclay.communication.grpc.logicmodule.NewNamespaceResponse\"\x00\x12\x87\x01\n\x0fremoveNamespace\x12?.dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x93\x01\n\x0egetNamespaceID\x12>.dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest\x1a?.dataclay.communication.grpc.logicmodule.GetNamespaceIDResponse\"\x00\x12\x99\x01\n\x10getNamespaceLang\x12@.dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest\x1a\x41.dataclay.communication.grpc.logicmodule.GetNamespaceLangResponse\"\x00\x12\x9f\x01\n\x12getObjectDataSetID\x12\x42.dataclay.communication.grpc.logicmodule.GetObjectDataSetIDRequest\x1a\x43.dataclay.communication.grpc.logicmodule.GetObjectDataSetIDResponse\"\x00\x12\x87\x01\n\x0fimportInterface\x12?.dataclay.communication.grpc.logicmodule.ImportInterfaceRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x85\x01\n\x0eimportContract\x12>.dataclay.communication.grpc.logicmodule.ImportContractRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xba\x01\n\x1bgetInfoOfClassesInNamespace\x12K.dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest\x1aL.dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse\"\x00\x12\xcc\x01\n!getImportedClassesInfoInNamespace\x12Q.dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest\x1aR.dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse\"\x00\x12\xa5\x01\n\x14getClassIDfromImport\x12\x44.dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest\x1a\x45.dataclay.communication.grpc.logicmodule.GetClassIDFromImportResponse\"\x00\x12\x90\x01\n\rgetNamespaces\x12=.dataclay.communication.grpc.logicmodule.GetNamespacesRequest\x1a>.dataclay.communication.grpc.logicmodule.GetNamespacesResponse\"\x00\x12\x87\x01\n\nnewDataSet\x12:.dataclay.communication.grpc.logicmodule.NewDataSetRequest\x1a;.dataclay.communication.grpc.logicmodule.NewDataSetResponse\"\x00\x12\x83\x01\n\rremoveDataSet\x12=.dataclay.communication.grpc.logicmodule.RemoveDataSetRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x8d\x01\n\x0cgetDataSetID\x12<.dataclay.communication.grpc.logicmodule.GetDataSetIDRequest\x1a=.dataclay.communication.grpc.logicmodule.GetDataSetIDResponse\"\x00\x12\xa5\x01\n\x14\x63heckDataSetIsPublic\x12\x44.dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicRequest\x1a\x45.dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicResponse\"\x00\x12\x9c\x01\n\x11getPublicDataSets\x12\x41.dataclay.communication.grpc.logicmodule.GetPublicDataSetsRequest\x1a\x42.dataclay.communication.grpc.logicmodule.GetPublicDataSetsResponse\"\x00\x12\x9f\x01\n\x12getAccountDataSets\x12\x42.dataclay.communication.grpc.logicmodule.GetAccountDataSetsRequest\x1a\x43.dataclay.communication.grpc.logicmodule.GetAccountDataSetsResponse\"\x00\x12\x81\x01\n\x08newClass\x12\x38.dataclay.communication.grpc.logicmodule.NewClassRequest\x1a\x39.dataclay.communication.grpc.logicmodule.NewClassResponse\"\x00\x12\x87\x01\n\nnewClassID\x12:.dataclay.communication.grpc.logicmodule.NewClassIDRequest\x1a;.dataclay.communication.grpc.logicmodule.NewClassIDResponse\"\x00\x12\x7f\n\x0bremoveClass\x12;.dataclay.communication.grpc.logicmodule.RemoveClassRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x87\x01\n\x0fremoveOperation\x12?.dataclay.communication.grpc.logicmodule.RemoveOperationRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x91\x01\n\x14removeImplementation\x12\x44.dataclay.communication.grpc.logicmodule.RemoveImplementationRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x93\x01\n\x0egetOperationID\x12>.dataclay.communication.grpc.logicmodule.GetOperationIDRequest\x1a?.dataclay.communication.grpc.logicmodule.GetOperationIDResponse\"\x00\x12\x90\x01\n\rgetPropertyID\x12=.dataclay.communication.grpc.logicmodule.GetPropertyIDRequest\x1a>.dataclay.communication.grpc.logicmodule.GetPropertyIDResponse\"\x00\x12\x87\x01\n\ngetClassID\x12:.dataclay.communication.grpc.logicmodule.GetClassIDRequest\x1a;.dataclay.communication.grpc.logicmodule.GetClassIDResponse\"\x00\x12\x8d\x01\n\x0cgetClassInfo\x12<.dataclay.communication.grpc.logicmodule.GetClassInfoRequest\x1a=.dataclay.communication.grpc.logicmodule.GetClassInfoResponse\"\x00\x12\x8a\x01\n\x0bnewContract\x12;.dataclay.communication.grpc.logicmodule.NewContractRequest\x1a<.dataclay.communication.grpc.logicmodule.NewContractResponse\"\x00\x12\x99\x01\n\x18registerToPublicContract\x12H.dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xd2\x01\n#registerToPublicContractOfNamespace\x12S.dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest\x1aT.dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceResponse\"\x00\x12\xb4\x01\n\x19getContractIDsOfApplicant\x12I.dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantRequest\x1aJ.dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse\"\x00\x12\xb1\x01\n\x18getContractIDsOfProvider\x12H.dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest\x1aI.dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse\"\x00\x12\xcc\x01\n%getContractIDsOfApplicantWithProvider\x12O.dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest\x1aP.dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse\"\x00\x12\x96\x01\n\x0fnewDataContract\x12?.dataclay.communication.grpc.logicmodule.NewDataContractRequest\x1a@.dataclay.communication.grpc.logicmodule.NewDataContractResponse\"\x00\x12\xa1\x01\n\x1cregisterToPublicDataContract\x12L.dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xc0\x01\n\x1dgetDataContractIDsOfApplicant\x12M.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantRequest\x1aN.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse\"\x00\x12\xbd\x01\n\x1cgetDataContractIDsOfProvider\x12L.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest\x1aM.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse\"\x00\x12\xdf\x01\n*getDataContractInfoOfApplicantWithProvider\x12V.dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest\x1aW.dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvResponse\"\x00\x12\x8d\x01\n\x0cnewInterface\x12<.dataclay.communication.grpc.logicmodule.NewInterfaceRequest\x1a=.dataclay.communication.grpc.logicmodule.NewInterfaceResponse\"\x00\x12\x99\x01\n\x10getInterfaceInfo\x12@.dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest\x1a\x41.dataclay.communication.grpc.logicmodule.GetInterfaceInfoResponse\"\x00\x12\x87\x01\n\x0fremoveInterface\x12?.dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xbd\x01\n\x1cgetExecutionEnvironmentsInfo\x12L.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest\x1aM.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse\"\x00\x12\xc0\x01\n\x1dgetExecutionEnvironmentsNames\x12M.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest\x1aN.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesResponse\"\x00\x12\xbd\x01\n\x1cgetExecutionEnvironmentForDS\x12L.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSRequest\x1aM.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSResponse\"\x00\x12\xae\x01\n\x17getStorageLocationForDS\x12G.dataclay.communication.grpc.logicmodule.GetStorageLocationForDSRequest\x1aH.dataclay.communication.grpc.logicmodule.GetStorageLocationForDSResponse\"\x00\x12\x90\x01\n\rgetObjectInfo\x12=.dataclay.communication.grpc.logicmodule.GetObjectInfoRequest\x1a>.dataclay.communication.grpc.logicmodule.GetObjectInfoResponse\"\x00\x12\x9f\x01\n\x12getObjectFromAlias\x12\x42.dataclay.communication.grpc.logicmodule.GetObjectFromAliasRequest\x1a\x43.dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse\"\x00\x12\x7f\n\x0b\x64\x65leteAlias\x12;.dataclay.communication.grpc.logicmodule.DeleteAliasRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xcf\x01\n\"getObjectsMetaDataInfoOfClassForNM\x12R.dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMRequest\x1aS.dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse\"\x00\x12y\n\x08\x61\x64\x64\x41lias\x12\x38.dataclay.communication.grpc.logicmodule.AddAliasRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x90\x01\n\x14registerObjectFromGC\x12\x43.dataclay.communication.grpc.logicmodule.RegisterObjectForGCRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x8b\x01\n\x11unregisterObjects\x12\x41.dataclay.communication.grpc.logicmodule.UnregisterObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x85\x01\n\x0eregisterObject\x12>.dataclay.communication.grpc.logicmodule.RegisterObjectRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xa9\x01\n setDataSetIDFromGarbageCollector\x12P.dataclay.communication.grpc.logicmodule.SetDataSetIDFromGarbageCollectorRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x81\x01\n\x0csetDataSetID\x12<.dataclay.communication.grpc.logicmodule.SetDataSetIDRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x87\x01\n\nnewVersion\x12:.dataclay.communication.grpc.logicmodule.NewVersionRequest\x1a;.dataclay.communication.grpc.logicmodule.NewVersionResponse\"\x00\x12\x8d\x01\n\x12\x63onsolidateVersion\x12\x42.dataclay.communication.grpc.logicmodule.ConsolidateVersionRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x87\x01\n\nnewReplica\x12:.dataclay.communication.grpc.logicmodule.NewReplicaRequest\x1a;.dataclay.communication.grpc.logicmodule.NewReplicaResponse\"\x00\x12\x87\x01\n\nmoveObject\x12:.dataclay.communication.grpc.logicmodule.MoveObjectRequest\x1a;.dataclay.communication.grpc.logicmodule.MoveObjectResponse\"\x00\x12\x8b\x01\n\x11setObjectReadOnly\x12\x41.dataclay.communication.grpc.logicmodule.SetObjectReadOnlyRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x8d\x01\n\x12setObjectReadWrite\x12\x42.dataclay.communication.grpc.logicmodule.SetObjectReadWriteRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x99\x01\n\x10getMetadataByOID\x12@.dataclay.communication.grpc.logicmodule.GetMetadataByOIDRequest\x1a\x41.dataclay.communication.grpc.logicmodule.GetMetadataByOIDResponse\"\x00\x12\xa8\x01\n\x15\x65xecuteImplementation\x12\x45.dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest\x1a\x46.dataclay.communication.grpc.logicmodule.ExecuteImplementationResponse\"\x00\x12\xa8\x01\n\x15\x65xecuteMethodOnTarget\x12\x45.dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest\x1a\x46.dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetResponse\"\x00\x12\x9d\x01\n\x1asynchronizeFederatedObject\x12J.dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x83\x01\n\rgetDataClayID\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a>.dataclay.communication.grpc.logicmodule.GetDataClayIDResponse\"\x00\x12\xb1\x01\n\x18registerExternalDataClay\x12H.dataclay.communication.grpc.logicmodule.RegisterExternalDataClayRequest\x1aI.dataclay.communication.grpc.logicmodule.RegisterExternalDataClayResponse\"\x00\x12\xd5\x01\n$notifyRegistrationOfExternalDataClay\x12T.dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest\x1aU.dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayResponse\"\x00\x12\xa4\x01\n\x17getExternalDataClayInfo\x12\x42.dataclay.communication.grpc.logicmodule.GetExtDataClayInfoRequest\x1a\x43.dataclay.communication.grpc.logicmodule.GetExtDataClayInfoResponse\"\x00\x12\xa8\x01\n\x15getExternalDataclayId\x12\x45.dataclay.communication.grpc.logicmodule.GetExternalDataclayIDRequest\x1a\x46.dataclay.communication.grpc.logicmodule.GetExternalDataclayIDResponse\"\x00\x12\x85\x01\n\x0e\x66\x65\x64\x65rateObject\x12>.dataclay.communication.grpc.logicmodule.FederateObjectRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x89\x01\n\x10unfederateObject\x12@.dataclay.communication.grpc.logicmodule.UnfederateObjectRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x99\x01\n\x18notifyUnfederatedObjects\x12H.dataclay.communication.grpc.logicmodule.NotifyUnfederatedObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x95\x01\n\x16notifyFederatedObjects\x12\x46.dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xe3\x01\n*checkObjectIsFederatedWithDataClayInstance\x12X.dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceRequest\x1aY.dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceResponse\"\x00\x12\xcc\x01\n!getDataClaysObjectIsFederatedWith\x12Q.dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithRequest\x1aR.dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithResponse\"\x00\x12\xcc\x01\n!getExternalSourceDataClayOfObject\x12Q.dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectRequest\x1aR.dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectResponse\"\x00\x12\x91\x01\n\x14unfederateAllObjects\x12\x44.dataclay.communication.grpc.logicmodule.UnfederateAllObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xa5\x01\n\x1eunfederateAllObjectsWithAllDCs\x12N.dataclay.communication.grpc.logicmodule.UnfederateAllObjectsWithAllDCsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x9d\x01\n\x1aunfederateObjectWithAllDCs\x12J.dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x97\x01\n\x17migrateFederatedObjects\x12G.dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x8d\x01\n\x12\x66\x65\x64\x65rateAllObjects\x12\x42.dataclay.communication.grpc.logicmodule.FederateAllObjectsRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x81\x01\n\x08getStubs\x12\x38.dataclay.communication.grpc.logicmodule.GetStubsRequest\x1a\x39.dataclay.communication.grpc.logicmodule.GetStubsResponse\"\x00\x12\x90\x01\n\rgetBabelStubs\x12=.dataclay.communication.grpc.logicmodule.GetBabelStubsRequest\x1a>.dataclay.communication.grpc.logicmodule.GetBabelStubsResponse\"\x00\x12\x7f\n\x0bregisterECA\x12;.dataclay.communication.grpc.logicmodule.RegisterECARequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x7f\n\x0b\x61\x64viseEvent\x12;.dataclay.communication.grpc.logicmodule.AdviseEventRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\x91\x01\n\x14isPrefetchingEnabled\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x45.dataclay.communication.grpc.logicmodule.IsPrefetchingEnabledResponse\"\x00\x12\x9c\x01\n\x11getClassNameForDS\x12\x41.dataclay.communication.grpc.logicmodule.GetClassNameForDSRequest\x1a\x42.dataclay.communication.grpc.logicmodule.GetClassNameForDSResponse\"\x00\x12\xc0\x01\n\x1dgetClassNameAndNamespaceForDS\x12M.dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSRequest\x1aN.dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse\"\x00\x12\xc6\x01\n\x1fgetContractIDOfDataClayProvider\x12O.dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderRequest\x1aP.dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderResponse\"\x00\x12\xab\x01\n\x16objectExistsInDataClay\x12\x46.dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayRequest\x1aG.dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayResponse\"\x00\x12\x81\x01\n\x0c\x63loseSession\x12<.dataclay.communication.grpc.logicmodule.CloseSessionRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12\xa8\x01\n\x15getMetadataByOIDForDS\x12\x45.dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSRequest\x1a\x46.dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSResponse\"\x00\x12\x87\x01\n\x0f\x61\x63tivateTracing\x12?.dataclay.communication.grpc.logicmodule.ActivateTracingRequest\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12z\n\x11\x64\x65\x61\x63tivateTracing\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12v\n\tgetTraces\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x35.dataclay.communication.grpc.common.GetTracesResponse\"\x00\x12|\n\x13\x63leanMetaDataCaches\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12w\n\x0e\x63loseManagerDb\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x12p\n\x07\x63loseDb\x12\x30.dataclay.communication.grpc.common.EmptyMessage\x1a\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x00\x42K\n1dataclay.communication.grpc.generated.logicmoduleB\x16LogicModuleGrpcServiceb\x06proto3')
  ,
  dependencies=[dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2.DESCRIPTOR,dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_LOGICMODULE = _descriptor.ServiceDescriptor(
  name='LogicModule',
  full_name='dataclay.communication.grpc.logicmodule.LogicModule',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=258,
  serialized_end=17806,
  methods=[
  _descriptor.MethodDescriptor(
    name='autoregisterSL',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.autoregisterSL',
    index=0,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._AUTOREGISTERSLREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='autoregisterEE',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.autoregisterEE',
    index=1,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._AUTOREGISTEREEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._AUTOREGISTEREERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unregisterStorageLocation',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unregisterStorageLocation',
    index=2,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNREGISTERSTORAGELOCATIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unregisterExecutionEnvironment',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unregisterExecutionEnvironment',
    index=3,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNREGISTEREXECUTIONENVIRONMENTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='checkAlive',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.checkAlive',
    index=4,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='performSetOfNewAccounts',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.performSetOfNewAccounts',
    index=5,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._PERFORMSETACCOUNTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._PERFORMSETACCOUNTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='performSetOfOperations',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.performSetOfOperations',
    index=6,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._PERFORMSETOPERATIONSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._PERFORMSETOPERATIONSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='publishAddress',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.publishAddress',
    index=7,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._PUBLISHADDRESSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newAccountNoAdmin',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newAccountNoAdmin',
    index=8,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWACCOUNTNOADMINREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWACCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newAccount',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newAccount',
    index=9,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWACCOUNTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWACCOUNTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getAccountID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getAccountID',
    index=10,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getAccountList',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getAccountList',
    index=11,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTLISTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTLISTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newSession',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newSession',
    index=12,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWSESSIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWSESSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInfoOfSessionForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getInfoOfSessionForDS',
    index=13,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINFOOFSESSIONFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINFOOFSESSIONFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newNamespace',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newNamespace',
    index=14,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWNAMESPACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWNAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeNamespace',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeNamespace',
    index=15,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVENAMESPACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getNamespaceID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getNamespaceID',
    index=16,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACEIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACEIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getNamespaceLang',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getNamespaceLang',
    index=17,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACELANGREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACELANGRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getObjectDataSetID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getObjectDataSetID',
    index=18,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTDATASETIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTDATASETIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='importInterface',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.importInterface',
    index=19,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._IMPORTINTERFACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='importContract',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.importContract',
    index=20,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._IMPORTCONTRACTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInfoOfClassesInNamespace',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getInfoOfClassesInNamespace',
    index=21,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINFOOFCLASSESINNAMESPACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINFOOFCLASSESINNAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getImportedClassesInfoInNamespace',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getImportedClassesInfoInNamespace',
    index=22,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getClassIDfromImport',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getClassIDfromImport',
    index=23,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSIDFROMIMPORTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSIDFROMIMPORTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getNamespaces',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getNamespaces',
    index=24,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACESREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETNAMESPACESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newDataSet',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newDataSet',
    index=25,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWDATASETREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWDATASETRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeDataSet',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeDataSet',
    index=26,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVEDATASETREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataSetID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataSetID',
    index=27,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATASETIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATASETIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='checkDataSetIsPublic',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.checkDataSetIsPublic',
    index=28,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CHECKDATASETISPUBLICREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CHECKDATASETISPUBLICRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getPublicDataSets',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getPublicDataSets',
    index=29,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETPUBLICDATASETSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETPUBLICDATASETSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getAccountDataSets',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getAccountDataSets',
    index=30,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTDATASETSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETACCOUNTDATASETSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newClass',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newClass',
    index=31,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCLASSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCLASSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newClassID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newClassID',
    index=32,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCLASSIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCLASSIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeClass',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeClass',
    index=33,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVECLASSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeOperation',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeOperation',
    index=34,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVEOPERATIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeImplementation',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeImplementation',
    index=35,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVEIMPLEMENTATIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getOperationID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getOperationID',
    index=36,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOPERATIONIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOPERATIONIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getPropertyID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getPropertyID',
    index=37,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETPROPERTYIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETPROPERTYIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getClassID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getClassID',
    index=38,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getClassInfo',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getClassInfo',
    index=39,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSINFOREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newContract',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newContract',
    index=40,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCONTRACTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWCONTRACTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerToPublicContract',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerToPublicContract',
    index=41,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTERTOPUBLICCONTRACTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerToPublicContractOfNamespace',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerToPublicContractOfNamespace',
    index=42,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getContractIDsOfApplicant',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getContractIDsOfApplicant',
    index=43,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDSOFAPPLICANTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDSOFAPPLICANTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getContractIDsOfProvider',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getContractIDsOfProvider',
    index=44,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDSOFPROVIDERREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDSOFPROVIDERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getContractIDsOfApplicantWithProvider',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getContractIDsOfApplicantWithProvider',
    index=45,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTSOFAPPLICANTWITHPROVREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newDataContract',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newDataContract',
    index=46,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWDATACONTRACTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWDATACONTRACTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerToPublicDataContract',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerToPublicDataContract',
    index=47,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTERTOPUBLICDATACONTRACTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataContractIDsOfApplicant',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataContractIDsOfApplicant',
    index=48,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTIDSOFAPPLICANTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTIDSOFAPPLICANTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataContractIDsOfProvider',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataContractIDsOfProvider',
    index=49,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTIDSOFPROVIDERREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTIDSOFPROVIDERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataContractInfoOfApplicantWithProvider',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataContractInfoOfApplicantWithProvider',
    index=50,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACONTRACTINFOOFAPPLICANTWITHPROVRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newInterface',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newInterface',
    index=51,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWINTERFACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWINTERFACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getInterfaceInfo',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getInterfaceInfo',
    index=52,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINTERFACEINFOREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETINTERFACEINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='removeInterface',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.removeInterface',
    index=53,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REMOVEINTERFACEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExecutionEnvironmentsInfo',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExecutionEnvironmentsInfo',
    index=54,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTSINFOREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTSINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExecutionEnvironmentsNames',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExecutionEnvironmentsNames',
    index=55,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTSNAMESREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTSNAMESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExecutionEnvironmentForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExecutionEnvironmentForDS',
    index=56,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXECUTIONENVIRONMENTFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getStorageLocationForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getStorageLocationForDS',
    index=57,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETSTORAGELOCATIONFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETSTORAGELOCATIONFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getObjectInfo',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getObjectInfo',
    index=58,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTINFOREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getObjectFromAlias',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getObjectFromAlias',
    index=59,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTFROMALIASREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTFROMALIASRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='deleteAlias',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.deleteAlias',
    index=60,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._DELETEALIASREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getObjectsMetaDataInfoOfClassForNM',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getObjectsMetaDataInfoOfClassForNM',
    index=61,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTSMETADATAINFOOFCLASSFORNMREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='addAlias',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.addAlias',
    index=62,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._ADDALIASREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerObjectFromGC',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerObjectFromGC',
    index=63,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTEROBJECTFORGCREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unregisterObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unregisterObjects',
    index=64,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNREGISTEROBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerObject',
    index=65,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTEROBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setDataSetIDFromGarbageCollector',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.setDataSetIDFromGarbageCollector',
    index=66,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._SETDATASETIDFROMGARBAGECOLLECTORREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setDataSetID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.setDataSetID',
    index=67,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._SETDATASETIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newVersion',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newVersion',
    index=68,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWVERSIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWVERSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='consolidateVersion',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.consolidateVersion',
    index=69,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CONSOLIDATEVERSIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='newReplica',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.newReplica',
    index=70,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWREPLICAREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NEWREPLICARESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='moveObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.moveObject',
    index=71,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._MOVEOBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._MOVEOBJECTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setObjectReadOnly',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.setObjectReadOnly',
    index=72,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._SETOBJECTREADONLYREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='setObjectReadWrite',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.setObjectReadWrite',
    index=73,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._SETOBJECTREADWRITEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getMetadataByOID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getMetadataByOID',
    index=74,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETMETADATABYOIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETMETADATABYOIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='executeImplementation',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.executeImplementation',
    index=75,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._EXECUTEIMPLEMENTATIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._EXECUTEIMPLEMENTATIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='executeMethodOnTarget',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.executeMethodOnTarget',
    index=76,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._EXECUTEMETHODONTARGETREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._EXECUTEMETHODONTARGETRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='synchronizeFederatedObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.synchronizeFederatedObject',
    index=77,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._SYNCHRONIZEFEDERATEDOBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataClayID',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataClayID',
    index=78,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACLAYIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerExternalDataClay',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerExternalDataClay',
    index=79,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTEREXTERNALDATACLAYREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTEREXTERNALDATACLAYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='notifyRegistrationOfExternalDataClay',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.notifyRegistrationOfExternalDataClay',
    index=80,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NOTIFYREGISTRATIONOFEXTERNALDATACLAYREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExternalDataClayInfo',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExternalDataClayInfo',
    index=81,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTDATACLAYINFOREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTDATACLAYINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExternalDataclayId',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExternalDataclayId',
    index=82,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTERNALDATACLAYIDREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTERNALDATACLAYIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='federateObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.federateObject',
    index=83,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._FEDERATEOBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unfederateObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unfederateObject',
    index=84,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNFEDERATEOBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='notifyUnfederatedObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.notifyUnfederatedObjects',
    index=85,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NOTIFYUNFEDERATEDOBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='notifyFederatedObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.notifyFederatedObjects',
    index=86,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._NOTIFYFEDERATEDOBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='checkObjectIsFederatedWithDataClayInstance',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.checkObjectIsFederatedWithDataClayInstance',
    index=87,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataClaysObjectIsFederatedWith',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getDataClaysObjectIsFederatedWith',
    index=88,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACLAYSOBJECTISFEDERATEDWITHREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getExternalSourceDataClayOfObject',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getExternalSourceDataClayOfObject',
    index=89,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTERNALSOURCEDATACLAYOFOBJECTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unfederateAllObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unfederateAllObjects',
    index=90,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNFEDERATEALLOBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unfederateAllObjectsWithAllDCs',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unfederateAllObjectsWithAllDCs',
    index=91,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNFEDERATEALLOBJECTSWITHALLDCSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='unfederateObjectWithAllDCs',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.unfederateObjectWithAllDCs',
    index=92,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._UNFEDERATEOBJECTWITHALLDCSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='migrateFederatedObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.migrateFederatedObjects',
    index=93,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._MIGRATEFEDERATEDOBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='federateAllObjects',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.federateAllObjects',
    index=94,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._FEDERATEALLOBJECTSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getStubs',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getStubs',
    index=95,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETSTUBSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETSTUBSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getBabelStubs',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getBabelStubs',
    index=96,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETBABELSTUBSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETBABELSTUBSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='registerECA',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.registerECA',
    index=97,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._REGISTERECAREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='adviseEvent',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.adviseEvent',
    index=98,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._ADVISEEVENTREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='isPrefetchingEnabled',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.isPrefetchingEnabled',
    index=99,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._ISPREFETCHINGENABLEDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getClassNameForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getClassNameForDS',
    index=100,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSNAMEFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSNAMEFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getClassNameAndNamespaceForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getClassNameAndNamespaceForDS',
    index=101,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSNAMEANDNAMESPACEFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCLASSNAMEANDNAMESPACEFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getContractIDOfDataClayProvider',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getContractIDOfDataClayProvider',
    index=102,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDOFDATACLAYPROVIDERREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='objectExistsInDataClay',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.objectExistsInDataClay',
    index=103,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._OBJECTEXISTSINDATACLAYREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._OBJECTEXISTSINDATACLAYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='closeSession',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.closeSession',
    index=104,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._CLOSESESSIONREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getMetadataByOIDForDS',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getMetadataByOIDForDS',
    index=105,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETMETADATABYOIDFORDSREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._GETMETADATABYOIDFORDSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='activateTracing',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.activateTracing',
    index=106,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_logicmodule_dot_logicmodule__messages__pb2._ACTIVATETRACINGREQUEST,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='deactivateTracing',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.deactivateTracing',
    index=107,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getTraces',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.getTraces',
    index=108,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._GETTRACESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='cleanMetaDataCaches',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.cleanMetaDataCaches',
    index=109,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='closeManagerDb',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.closeManagerDb',
    index=110,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='closeDb',
    full_name='dataclay.communication.grpc.logicmodule.LogicModule.closeDb',
    index=111,
    containing_service=None,
    input_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EMPTYMESSAGE,
    output_type=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_LOGICMODULE)

DESCRIPTOR.services_by_name['LogicModule'] = _LOGICMODULE

# @@protoc_insertion_point(module_scope)




""" Class description goes here. """

from dataclay.util.MgrObject import ManagementObject
from dataclay.util.management.classmgr.Type import Type


# Added implPosition
class ImplementationStubInfo(ManagementObject):
    _fields = ["namespace",
               "className",
               "signature",
               "params",
               "paramsOrder",
               "returnType",
               "namespaceID",
               "operationID",
               "localImplID",
               "remoteImplID",
               "contractID",
               "interfaceID",
               "responsibleRemoteAccountID",
               "implPosition",
               ]

    _internal_fields = list()

    _typed_fields = {"returnType": Type}





""" Class description goes here. """

import os.path

from dataclay.util.PropertiesFilesLoader import PropertyFile

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'


class ErrorCodes(PropertyFile):
    """Property holder for the "errorcodes.properties" file."""

    def __init__(self, property_path, error_codes_name="errorcodes.properties"):
        self.error_codes = {}
        super(ErrorCodes, self).__init__(os.path.join(property_path, error_codes_name))

    def _process_line(self, key, value):
        """The error codes are ints, and reverse lookup is useful."""
        error_code = int(value)
        self.__dict__[key] = error_code
        self.error_codes[error_code] = key



from dataclay import DataClayObject, dclayMethod


class StorageDict(DataClayObject):
    """Demonstration of the StorageDict class.

    This class is prepared for demonstration purposes and is not suitable,
    as it is right now, for HPC applications and production workflows.

    @ClassField _dict anything
    """

    @dclayMethod()
    def __init__(self):
        self._dict = dict()

    @dclayMethod(return_="int")
    def __len__(self):
        return len(self._dict)

    @dclayMethod(key="anything", return_="anything")
    def __getitem__(self, key):
        return self._dict[key]

    @dclayMethod(key="anything", value="anything")
    def __setitem__(self, key, value):
        self._dict[key] = value

    @dclayMethod(key="anything")
    def __delitem__(self, key):
        del self._dict[key]

    @dclayMethod(_local=True, return_="anything")
    def __iter__(self):
        return iter(self._dict)

    @dclayMethod(key="anything", return_="bool")
    def __contains__(self, key):
        return key in self._dict

    @dclayMethod(_local=True, return_="anything")
    def keys(self):
        return self._dict.keys()

    @dclayMethod(_local=True, return_="anything")
    def items(self):
        return self._dict.items()

    @dclayMethod(_local=True, return_="anything")
    def values(self):
        return self._dict.values()

    @dclayMethod(return_="anything")
    def split(self):
        # Ugly split-in-two, for demonstration purposes only
        from itertools import cycle

        out_a = dict()
        out_b = dict()

        for (key, value), out in zip(self._dict.items(), cycle(out_a, out_b)):
            out[key] = value

        return (out_a, out_b)

    @dclayMethod(return_="str")
    def __str__(self):
        return str(self._dict)




""" Class description goes here. """

from dataclay.serialization.DataClaySerializable import DataClaySerializable

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'


class DataClayPythonWrapper(DataClaySerializable):
    __slots__ = ()




""" Class description goes here. """

from dataclay.util.MgrObject import ManagementObject


class Implementation(ManagementObject):
    _fields = ["dataClayID",
               "responsibleAccountName",
               "namespace",
               "className",
               "opNameAndDescriptor",
               "position",
               "includes",
               "accessedProperties",
               "accessedImplementations",
               "requiredQuantitativeFeatures",
               "requiredQualitativeFeatures",
               ]

    _internal_fields = ["operationID",
                        "metaClassID",
                        "responsibleAccountID",
                        "namespaceID",
                        "prefetchingInfo",
                        ]




""" Class description goes here. """

"""Serialization code related to DataClay objects

The code implemented in this module is (at "this moment") identical to the ones
implemented in the Java package client.CommonLib. Specifically, the serialize*
functions are more or less adapted here.
"""
from io import BytesIO
import logging

import dataclay
from dataclay.exceptions.exceptions import InvalidPythonSignature
from dataclay.serialization.python.lang.VLQIntegerWrapper import VLQIntegerWrapper
from dataclay.serialization.python.util.PyTypeWildcardWrapper import PyTypeWildcardWrapper, safe_wait_if_compss_future
from dataclay.util.IdentityDict import IdentityDict
import dataclay.communication.grpc.messages.common.common_messages_pb2 as common_messages
from dataclay.communication.grpc.Utils import get_metadata
from dataclay.util.ReferenceCounting import ReferenceCounting

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'

logger = logging.getLogger(__name__)


class SerializationLibUtils(object):

    def _create_buffer_and_serialize(self, instance, ignore_user_types, ifacebitmaps, cur_serialized_objs, pending_objs,
                                     reference_counting, return_none_if_no_ref_counting=False):
        """
        @postcondition: Create buffer and serialize. TODO: modify buffer
        @param instance: Instance to serialize
        @param ignore_user_types: Indicates if user types found during serialization must be ignored or not
        @param ifacebitmaps: Map of bitmaps representing the interfaces to use
        @param cur_serialized_objs: Current serialized objects Object -> OID tag. This structure must be different during each serialization since OID
                tags are not shared.
        @param pending_objs: pending objects
        @param reference_counting: reference counting
        @param return_none_if_no_ref_counting: If true, return null if object does not reference any other object except language object. This is useful during
                   GC of not dirty objects without references.
        @return serialized bytes
        """

        buffer = BytesIO()
        instance.serialize(buffer, ignore_user_types, None, cur_serialized_objs, pending_objs, reference_counting)
        if return_none_if_no_ref_counting:
            if reference_counting.has_no_references():
                return None
        metadata = self.create_metadata(cur_serialized_objs, None, 0)
        dcc_extradata = instance.get_class_extradata()
        return instance.get_object_id(), dcc_extradata.class_id, metadata, buffer

    def serialize_association(self,
            io_output,  # final DataClayByteBuffer
            element,  # final DataClayObject
            cur_serialized_objs,  # final IdentityHashMap<Object, Integer>
            pending_objs,  # ListIterator<DataClayObject>
            reference_counting
):
        logger.verbose("Found association %s", element)
        try:
            tag = cur_serialized_objs[element]
        except KeyError:
            logger.debug("Adding object %s to pending_objects", element.get_object_id())

            pending_objs.append(element)
            tag = len(cur_serialized_objs)
            cur_serialized_objs[element] = tag

        """ update reference counting """
        associated_oid = element.get_object_id()
        hint = element.get_hint()
        reference_counting.increment_reference_counting(associated_oid, hint)

        """ write tag """
        VLQIntegerWrapper().write(io_output, tag)

    def serialize_params_or_return(self, params, iface_bitmaps, params_spec,
            params_order, hint_volatiles, runtime, recursive=True, for_update=False):

        """
        @postcondition: serialize parameters or return
        @param params: param
        :returns: serialized params [0: num_params, 1: imm_objs, 2: lang_objs, 3: vol_params, 4:pers_params] with following structure
            imm_objs is a dict of idx -> bytes
            lang_objs is a dict of idx -> bytes
            vol_params is a dict of object id -> [0: object id, 1: class id, 2: metadata, 3: bytes]
            pers_params is a dict of object id -> [0: oid, 1: hint, 2: class_id, 3:False ]
        """

        pending_objects = list()
        already_serialized_params = set()

        imm_objs = dict()
        lang_objs = dict()
        vol_params = dict()
        pers_params = dict()

        i = 0
        num_params = 0
        if params_order is not None:
            num_params = len(params_order)
        if num_params > 0:
            for param_name in params_order:

                param = params[i]
                if param is not None:
                    param = safe_wait_if_compss_future(param)

                    try:
                        oid = param.get_object_id()

                        runtime.add_session_reference(oid)

                        if param.is_persistent():
                            logger.debug("Serializing persistent parameter/return with oid %s", oid)

                            class_id = param.get_class_extradata().class_id
                            hint = param.get_hint()

                            pers_param = [oid, hint, class_id, False]
                            pers_params[i] = pers_param

                        else:
                            logger.debug("Serializing volatile parameter/return with oid %s", oid)

                            # this is no-exception flow which means...
                            # ... means that it is a volatile object to serialize, with its OID
                            already_serialized_params.add(oid)
                            obj_with_data = self.serialize_dcobj_with_data(
                                        param,  # final DataClayObject
                                        pending_objects,  # final ListIterator<DataClayObject>
                                        not recursive,
                                        hint_volatiles,
                                        runtime, True, for_update)

                            vol_params[i] = obj_with_data

                    except AttributeError:
                        try:
                            # ToDo: support for notifications (which is said to leave params_spec None)
                            param_type = params_spec[param_name]
                            ptw = PyTypeWildcardWrapper(param_type.signature)
                        except InvalidPythonSignature:
                            raise NotImplementedError("In fact, InvalidPythonSignature was "
                                                      "not even implemented, seems somebody is "
                                                      "raising it without implementing logic.")
                        else:
                            io_output = BytesIO()
                            ptw.write(io_output, param)
                            imm_objs[i] = io_output
                i = i + 1

            if recursive:
                while pending_objects:
                    # Note that pending objects are *only* DataClay Objects)
                    pending_obj = pending_objects.pop()
                    oid = pending_obj.get_object_id()

                    if oid in already_serialized_params:
                        continue

                    if pending_obj.is_persistent():
                        logger.debug("Serializing sub-object persistent parameter/return with oid %s", oid)

                        class_id = param.get_class_extradata().class_id
                        hint = param.get_hint()

                        pers_param = [oid, hint, class_id, False]
                        pers_params[i] = pers_param
                    else:
                        logger.debug("Serializing sub-object volatile parameter/return with oid %s", oid)

                        obj_with_data = self.serialize_dcobj_with_data(
                                                pending_obj,
                                                pending_objects,
                                                not recursive,
                                                hint_volatiles, runtime, True, for_update)

                        vol_params[i] = obj_with_data

                    # Ensure that it is put inside
                    already_serialized_params.add(oid)
                    i += 1
        else:
            logger.debug("Call with no parameters, no serialization required")

        for vol in vol_params.values():
            logger.debug("Serialized volatile: %s", vol)
        for persi in pers_params.values():
            logger.debug("Serialized persistent param: %s", persi)

        serialized_params = [num_params, imm_objs, lang_objs, vol_params, pers_params]

        return serialized_params

    def serialize_dcobj_with_data(self, dc_object, pending_objs, ignore_user_types, hint, runtime,
                                  force_pending_to_register, for_update=False):

        """
        @postcondition: Serialize DataClayObject with data.
        @param dc_object: DCObject
        @param ignore_user_types: Indicates if user types inside the instance must be ignored or not
        @param pending_objs: pending objects
        @param hint: Hint to set if needed
        @param runtime; Runtime managed
        @param force_pending_to_register: If TRUE, object is going to be set as pending to register. Take into account that this function is also called to
            serialize objects in 'moves','replicas',... and in that case this parameter must be FALSE to avoid overriding the
            actual value of the instance (actual pending or not).
        @param for_update whether this serialziation comes from an update operation or not
        """

        object_with_data = None
        object_id = dc_object.get_object_id()
        runtime.lock(object_id)
        try:
            cur_serialized_objs = IdentityDict()
            reference_counting = ReferenceCounting()
            cur_serialized_objs[dc_object] = 0
            """ Set flags of volatile/persisted obj. being send """
            # Set Hint in object being send/persisted. Hint provided during creation of metadata
            # is for associated objects that are not persistent yet (currently being persisted)
            # This algorithm can be improved in both languages, Python and Java.
            if for_update is False:
                dc_object.set_hint(hint)
                dc_object.set_persistent(True)
                
            object_with_data = self._create_buffer_and_serialize(dc_object, ignore_user_types, None, cur_serialized_objs, pending_objs, reference_counting) 

            if force_pending_to_register:
                dc_object.set_pending_to_register(True)

        finally:
            runtime.unlock(object_id)
        return object_with_data

    def create_metadata(self, cur_ser_objs, hint_for_missing, num_refs_pointing_to_obj):

        # Prepare metadata structure

        tags_to_oids = dict()
        tags_to_class_id = dict()
        tags_to_hint = dict()

        for k, v in cur_ser_objs.items():
            from dataclay import DataClayObject
            if isinstance(k, DataClayObject):

                obj = k
                tag = v
                class_id = obj.get_class_extradata().class_id
                object_id = obj.get_object_id()
                hint = obj.get_hint()

                tags_to_oids[tag] = object_id
                tags_to_class_id[tag] = class_id
                logger.debug("[==Object Metadata==] Adding metadata tag %s -> oid %s", tag, object_id)
                logger.debug("[==Object Metadata==] Adding metadata tag %s -> class id %s", tag, class_id)

                if hint is not None:
                    logger.debug("[==Hint==] Setting hint %s association for tag %s", hint, tag)
                    tags_to_hint[tag] = hint
                else:
                    if hint_for_missing is not None:
                        logger.debug("[==Hint==] Setting hint %s association for tag %s", hint_for_missing, tag)
                        tags_to_hint[tag] = hint_for_missing

        response = tags_to_oids, tags_to_class_id, tags_to_hint, num_refs_pointing_to_obj

        return response

    def serialize_for_db(self, object_id, metadata, object_bytes, is_store):
        """
        @postcondition: serialize object for DB store or update
        @param object_id: id of the object to store or update
        @param object_bytes: object bytes
        @param is_store: indicates if it is for a store in DB
        @return serialized msg
        """

        msg = common_messages.PersistentObjectInDB(data=object_bytes.getvalue(), metadata=get_metadata(metadata))
        msgstr = msg.SerializeToString()
        return msgstr

    def serialize_for_db_gc(self, instance, ignore_user_types, ifacebitmaps, return_none_if_no_ref_counting=False):
        """
        @postcondition: Serialize for flushing from GC
        @param instance: Instance to serialize
        @param ignore_user_types:Indicates if user types must be ignored
        @param ifacebitmaps: Interface bitmaps
        @param return_none_if_no_ref_counting: If true, return null if object does not reference any other object except language object. This is useful during
                   GC of not dirty objects without references.
        @return serialized bytes
        """

        cur_serialized_objs = IdentityDict()
        reference_counting = ReferenceCounting()
        pending_objs = list()
        cur_serialized_objs[instance] = 0
        obj_data = self._create_buffer_and_serialize(instance, ignore_user_types, ifacebitmaps, cur_serialized_objs, pending_objs,
                                                reference_counting, return_none_if_no_ref_counting=return_none_if_no_ref_counting)
        if return_none_if_no_ref_counting:
            if obj_data == None:
                return None
        return self.serialize_for_db(instance.get_object_id(), obj_data[2], obj_data[3], False)

    def serialize_for_db_gc_not_dirty(self, instance, ignore_user_types, ifacebitmaps, return_none_if_no_ref_counting=True):
        """
        @postcondition: Serialize for flushing from GC not-dirty objects
        @param instance: Instance to serialize
        @param ignore_user_types:Indicates if user types must be ignored
        @param ifacebitmaps: Interface bitmaps
        @param return_none_if_no_ref_counting: If true, return null if object does not reference any other object except language object. This is useful during
                   GC of not dirty objects without references.
        @return serialized bytes
        """

        cur_serialized_objs = IdentityDict()
        reference_counting = ReferenceCounting()
        pending_objs = list()
        cur_serialized_objs[instance] = 0
        obj_data = self._create_buffer_and_serialize(instance, ignore_user_types, ifacebitmaps, cur_serialized_objs, pending_objs,
                                                reference_counting, return_none_if_no_ref_counting=return_none_if_no_ref_counting)
        if return_none_if_no_ref_counting:
            if obj_data == None:
                return None
        return obj_data[3]


SerializationLibUtilsSingleton = SerializationLibUtils()


class PersistentIdPicklerHelper(object):
    """Helper to solve serialization of associations inside Pickled structures.

    See https://docs.python.org/2.7/library/pickle.html#pickling-and-unpickling-external-objects
    for more information of what this is doing. Note that the `__call__` is
    being called by Pickle for every object, and when we find a DataClayObject
    there, we proceed to do something very close to the serialize_association
    procedure.
    """

    def __init__(self, cur_serialized_objs, pending_objs, reference_counting):
        self._cur_serialized_objs = cur_serialized_objs
        self._pending_objs = pending_objs
        self._reference_counting = reference_counting

    def __call__(self, obj):
        if isinstance(obj, dataclay.DataClayObject):
            logger.verbose("Found association during Pickle process: %r", obj)
            try:
                tag = self._cur_serialized_objs[obj]
            except KeyError:
                logger.debug("Adding object %r to pending_objects", obj)

                self._pending_objs.append(obj)
                tag = len(self._cur_serialized_objs)
                self._cur_serialized_objs[obj] = tag

            """ update reference counting """
            associated_oid = obj.get_object_id()
            hint = obj.get_hint()
            self._reference_counting.increment_reference_counting(associated_oid, hint)

            return str(tag)





""" Class description goes here. """

from lru import LRU

from dataclay.util.MgrObject import ManagementObject
from .Type import Type
from .Utils import STATIC_ATTRIBUTE_FOR_EXTERNAL_INIT, stub_only_def, py_code


# Modified abstract with isAbstract, deleted dcID and implID as internal_fields
class MetaClass(ManagementObject):
    _fields = ["dataClayID",
               "namespace",
               "name",
               "parentType",
               "properties",
               "operations",
               "isAbstract",
               "languageDepInfos",
               "ecas"
               ]

    _internal_fields = ["namespaceID",
                        # Internal memoization for get_operation method
                        "_implementation_id_to_operation_cache", ]
    
    _typed_fields = {"parentType": Type}
        
    def get_operation_from_name(self, op_name):
        """Return the operation from its name."""
        # TODO: This implementation is slow, and is called from critical path.

        for op in self.operations:
            if op.name == op_name:
                return op

        raise KeyError("Operation with name %s was not found in dataClay class %s" % 
                       (op_name, self.name))

    def get_operation(self, implementation_id):
        """Return an Operation (management object) from an ImplementationID

        :param uuid.UUID implementation_id: The requested ImplementationID

        Given the UUID for a certain Implementation, lookup and return the
        corresponding Operation. Note that this method is memoized (cached) in
        order to improve performance (given that the lookup is slow).
        """
        if not hasattr(self, "_implementation_id_to_operation_cache"):
            # TODO: remove this hardcoded value for the LRU
            setattr(self, "_implementation_id_to_operation_cache", LRU(50))

        # If the key is in there, easy!
        try:
            return self._implementation_id_to_operation_cache[implementation_id]
        except KeyError:
            pass

        # Bad luck! Seems we must do the expensive lookup now
        for op in self.operations:
            for imp in op.implementations:
                if imp.dataClayID == implementation_id:
                    # Found it, store it in the LRU and also return it
                    self._implementation_id_to_operation_cache[implementation_id] = op
                    return op

        raise KeyError("Operation for ImplementationID {%s} not found in class %s (in namespace %s)" % 
                       (implementation_id, self.name, self.namespace))

    def juxtapose_code(self, exeenv_flag=False):
        """Return the complete source code for the current MetaClass.

        :param exeenv_flag: Set to true to generate code for the ExecutionEnvironment.
        :return: A valid source for this class.

        Note that this class will use the "local_implementation" of its
        operations > implementations when available. Undefined behaviour if
        not available.

        In scenarios where implementations are *not* `PythonImplementation` a
        pure-stub (intended for only persistent work mode) function is used. If
        the constructor is one of those non-Python methods, then the class is
        flagged as EXTERNAL_INIT only (see ExecutionGateway for further info).
        """
        from dataclay.util.management.classmgr.python.PythonImplementation import PythonImplementation

        imp_codes = list()

        # Java may have multiple overloads, ignore them
        ops_done = set()

        for op in self.operations:
            if op.name.startswith("$$"):
                # logger.debug("Ignoring method %s because it is a getter/setter", op.name)
                continue

            if op.name in ops_done:
                # logger.info("I may have found an overload for operation %s --ignoring", op.name)
                continue
            ops_done.add(op.name)

            if len(op.implementations) != 1:
                raise NotImplementedError("Found %d operations, but currently"
                                          " I only support one" % op.implementations)
            imp = op.implementations[0]

            if isinstance(imp, PythonImplementation):
                # ToDo fix behaviour regarding exeenv_flag, local/remote implementation
                imp_codes.append(imp.code)
            else:
                if op.name == "__init__" or op.name == "<init>":
                    imp_codes.append("\n    %s = %s" % (
                        STATIC_ATTRIBUTE_FOR_EXTERNAL_INIT, str(True)))
                    # Override the name, because java's <init> should become __init__
                    op.name = "__init__"

                imp_codes.append(stub_only_def.render({
                    "func_name": op.name,
                    "param_names": op.paramsOrder,
                }))

        return py_code.render({
            "class_name": self.name.rsplit('.', 1)[-1],
            "parent_name": self.parentType.typeName if self.parentType else "DataClayObject",
            "metaclass": self,
            "imp_codes": imp_codes,
        })
        
        # Added descriptor and nameAndDescriptor, modified isAbstract/isStaticConstructor, deleted dataclayID




""" Class description goes here. """

from dataclay.util.MgrObject import ManagementObject


class SessionInfo(ManagementObject):
    _fields = ["sessionID",
               "accountID",
               "propertiesOfClasses",
               "sessionContracts",
               "sessionDataContracts",
               "dataContractIDforStore",
               "language",
               "ifaceBitmaps",
               "endDate",
               ]

    _internal_fields = ["extDataClayID",
               ]



""" Class description goes here. """

import logging
import time
import uuid
import datetime
from dataclay.communication.grpc.messages.common.common_messages_pb2 import LANG_PYTHON
from dataclay.exceptions.exceptions import DataClayException
from dataclay.commonruntime.DataClayRuntime import DataClayRuntime
from dataclay.commonruntime.RuntimeType import RuntimeType
from dataclay.commonruntime.Settings import settings
from dataclay.serialization.lib.SerializationLibUtils import SerializationLibUtilsSingleton
from dataclay.heap.ExecutionEnvironmentHeapManager import ExecutionEnvironmentHeapManager
from dataclay.loader.ExecutionObjectLoader import ExecutionObjectLoader
from dataclay.commonruntime.Runtime import threadLocal
from dataclay.util import Configuration

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'

logger = logging.getLogger(__name__)

current_milli_time = lambda: int(round(time.time() * 1000))


class ExecutionEnvironmentRuntime(DataClayRuntime):
    
    def __init__(self, theexec_env):
        DataClayRuntime.__init__(self)
            
        self.current_type = RuntimeType.exe_env
    
        """ Execution Environment using this runtime. """
        self.execution_environment = theexec_env
        
        """
        References hold by sessions. Resource note: Maximum size of this map is maximum number of objects allowed in EE x sessions.
        Also, important to think what happens if one single session is associated to two client threads? use case? 
        should we allow that?
        Must be thread-safe.
        """
        self.references_hold_by_sessions = dict() 
    
        """
        Sessions in quarantine. note: maximum size of this map is max number of sessions per EE: This map is needed to solve a race
        condition in Global Garbage collection (@see getReferenceCounting). 
        """
        self.quarantine_sessions = set()
        
        """
        Per each session, it's expiration date. This is used to control 'retained' objects from sessions in Garbage collection.
        Must be thread-safe.
        """
        self.session_expires_dates = dict()
    
        """
        Alias references found. TODO: this set is not cleaned. Important: modify this when design of removeAlias: we need to add
        'alias' as DataClayObject field since alias can be added dynamically to a volatile. Or check updateAliases function. Also,
        ensure removeAlias in LM notifies EE and avoid race condition remove alias but continue using object and GlobalGC clean it.
        Should be thread-safe.
        """
        self.alias_references = set()

    def initialize_runtime_aux(self):
        self.dataclay_heap_manager = ExecutionEnvironmentHeapManager(self)
        self.dataclay_object_loader = ExecutionObjectLoader(self)
        
    def is_exec_env(self):
        return True

    def get_or_new_instance_from_db(self, object_id, retry):
        """
        @postcondition: Get object from memory or database and WAIT in case we are still waiting for it to be persisted.
        @param object_id: ID of object to get or create 
        @param retry: indicates if we should retry and wait 
        """ 
        return self.dataclay_object_loader.get_or_new_instance_from_db(object_id, retry)

    def load_object_from_db(self, instance, retry):
        """
        @postcondition: Load DataClayObject from Database
        @param instance: DataClayObject instance to fill
        @param retry: Indicates retry loading in case it is not in db.
        """
        return self.dataclay_object_loader.load_object_from_db(instance, retry)
    
    def get_hint(self):
        """
        @postcondition: Get hint of the current EE 
        @return Hint of current EE
        """
        return settings.environment_id

    def flush_all(self):
        """
        @postcondition: Flush all objects in memory to disk.
        """
        self.dataclay_heap_manager.flush_all()
    
    def get_session_id(self):
        """
        @postcondition: Get Session ID associated to current thread 
        @return: Session ID associated to current thread 
        """
        return threadLocal.session_id
    
    def get_execution_environment(self):
        """
        @return: Return execution environment using this runtime 
        """ 
        return self.execution_environment

    def store_object(self, instance):
        if not instance.is_persistent():
            raise RuntimeError("StoreObject should only be called on Persistent Objects. "
                               "Ensure to call make_persistent first")
    
        self.internal_store(instance, make_persistent=False)
    
    def make_persistent(self, instance, alias, backend_id, recursive):
        """ This method creates a new Persistent Object using the provided stub
        instance and, if indicated, all its associated objects also Logic module API used for communication
        This function is called from a stub/execution class
        :param instance: Instance to make persistent
        :param backend_id: Indicates which is the destination backend
        :param recursive: Indicates if make persistent is recursive
        :param alias: Alias for the object
        :returns: ID of the backend in which te object was persisted.
        :type instance: DataClayExecutionObject
        :type backend_id: DataClayID
        :type recursive: boolean
        :type alias: string
        :rtype: DataClayID
        :raises RuntimeError: if backend id is UNDEFINED_LOCAL.
        """
        self.logger.debug("Starting make persistent object for instance %s with id %s", instance,
                     instance.get_object_id())
        if alias is not None:
            # Add a new alias to an object.
            # We call 'addAlias' with registration information in case we need to register it.
            # Use cases:
            # 1 - object was persisted without alias and not yet registered -> we need to register it with new alias.
            # 2 - object was persisted and it is already registered -> we only add a new alias
            # 3 - object was persisted with an alias and it must be already registered -> we add a new alias.
            if instance.is_pending_to_register():
                # Use case 1
                reg_info = [instance.get_object_id(), instance.get_class_extradata().class_id,
                            self.get_session_id(), instance.get_dataset_id()]
                # TODO: Review if we use hint of the object or the hint of the runtime.
                self.ready_clients["@LM"].register_object(reg_info, instance.get_hint(), alias, LANG_PYTHON)
            else:
                # Use case 2 and 3 - add new alias
                self.ready_clients["@LM"].add_alias(instance.get_object_id(), alias)
            
        return instance.get_location()

    def execute_implementation_aux(self, operation_name, instance, parameters, exeenv_id=None):

        object_id = instance.get_object_id()
        
        logger.debug("Calling execute_implementation inside EE for operation %s and object id %s", operation_name, object_id)
    
        # # ============================== PARAMS/RETURNS ========================== //
        # # Check if object is being deserialized (params/returns)
        under_deserialization = self.check_and_fill_volatile_under_deserialization(instance, None)

        if under_deserialization:
            logger.debug("Object %s is a volatile under deserialization, executing", object_id)
            return self.execution_environment.internal_exec_impl(operation_name, instance, parameters) 
        
        # // === HINT === //
        thisExecEnv = settings.environment_id
        using_hint = False
        if exeenv_id is None:
            if instance.get_hint() is not None:
                exeenv_id = instance.get_hint()
                using_hint = True
                logger.debug("Using hint %s for object id %s", exeenv_id, object_id)
            else:
                logger.debug("Asking for EE of object with id %s", object_id)
                exeenv_id = next(iter(self.get_metadata(object_id).locations))
        
        if exeenv_id == thisExecEnv:        
            logger.debug("Object execution is local")
    
            # Note that fat_instance tend to be the same as instance...
            # *except* if it is a proxy
            try:
                fat_instance = self.execution_environment.get_local_instance(object_id)
                # get_local_instance should indeed modify the same instance instance,
                # so @abarcelo is leaving the assert just in case
                assert instance is fat_instance, \
                    "A tiny mess with get_local_instance and heap management, check that"
                return self.execution_environment.internal_exec_impl(operation_name, fat_instance, parameters) 
            except Exception as e:
                return self.execution_environment.internal_exec_impl(operation_name, instance, parameters) 
        else:
            logger.debug("Object execution is not local")
            object_id = instance.get_object_id()
            return self.call_execute_to_ds(instance, parameters, operation_name, exeenv_id, using_hint)
    
    #########################################
    # Helper functions, not commonruntime methods #
    #########################################
    
    def internal_store(self, instance, make_persistent=True):
        """Perform the storage (StoreObject call) for an instance.
    
        :param instance: The DataClayObject willing to be stored.
        :param make_persistent: Flag, True when DS_STORE_OBJECT should be called
        and False when DS_UPSERT_OBJECT is the method to be called.
        :return: A dictionary containing the classes for all stored objects.
    
        This function works for two main scenarios: the makePersistent one (in
        which the instance is not yet persistent) and the update (in which the
        instance is persistent).
    
        The return dictionary is the same in both cases, but note that the update
        should not use the provided instance for updating metadata to the LM.
        """
        logger.debug("Store procedure for instance %r", instance)
        client = self.ready_clients["@STORAGE"]
        
        pending_objs = [instance]
        stored_objects_classes = dict()
        serialized_objs = list()
        reg_infos = list()
        dataset_id = self.execution_environment.thread_local_info.dataset_id
    
        while pending_objs:
            current_obj = pending_objs.pop()
            # Lock and make sure it is loaded 
            current_obj_id = current_obj.get_object_id()
            self.lock(current_obj_id)  # Avoid GC clean object while storing it
            try:
                if not current_obj.is_loaded():
                    current_obj = self.get_or_new_instance_from_db(current_obj_id, False)
    
                dcc_extradata = current_obj.get_class_extradata()
                object_id = current_obj.get_object_id()
    
                if make_persistent:
                    # Ignore already persistent objects
                    if current_obj.is_persistent():
                        continue
    
                    dcc_extradata = current_obj.get_class_extradata()
                    infos = [object_id, dcc_extradata.class_id,
                             self.execution_environment.thread_local_info.session_id, dataset_id]
                    reg_infos.append(infos)
    
                # This object will soon be persistent
                current_obj.set_persistent(True)
                current_obj.set_hint(settings.environment_id)
                # Just in case (should have been loaded already)
                logger.debug("Setting loaded to true from internal store for object %s" % str(object_id))
                current_obj.set_loaded(True)
    
                # First store since others OIDs are recursively created while creating MetaData
                if not object_id:
                    if not make_persistent:
                        raise DataClayException("Objects should never be uuid-less for non-make_persistent use cases")
                    object_id = uuid.uuid4()
                    current_obj.set_object_id(object_id)
                    current_obj.set_dataset_id(self.execution_environment.thread_local_info.dataset_id)
        
                logger.debug("Ready to make persistent object {%s} of class %s {%s}" % 
                             (object_id, dcc_extradata.classname, dcc_extradata.class_id))
        
                stored_objects_classes[object_id] = dcc_extradata.class_id
        
                # If we are not in a make_persistent, the dataset_id hint is null (?)
                serialized_objs.append(SerializationLibUtilsSingleton.serialize_dcobj_with_data(
                    current_obj, pending_objs, False, None, self, False))
            finally:
                self.unlock(current_obj_id)
        
        if make_persistent:
            lm_client = self.ready_clients["@LM"]
            lm_client.register_objects(reg_infos, settings.environment_id, None, None,
                                           LANG_PYTHON)
            client.ds_store_objects(self.execution_environment.thread_local_info.session_id, serialized_objs, False, None)
        else:
            client.ds_upsert_objects(self.execution_environment.thread_local_info.session_id, serialized_objs)

    def get_operation_info(self, object_id, operation_name):
        dcc_extradata = self.get_object_by_id(object_id).get_class_extradata()
        metaclass_container = dcc_extradata.metaclass_container
        operation = metaclass_container.get_operation_from_name(operation_name)
        return operation 
    
    def get_implementation_id(self, object_id, operation_name, implementation_idx=0):
        operation = self.get_operation_info(object_id, operation_name)
        return operation.implementations[0].dataClayID
        
    def check_and_fill_volatile_under_deserialization(self, volatile_obj, ifacebitmaps):
        """ Check if there is a volatile object with ID provided pending to deserialize and if so, deserialize it since it is needed.
        :param volatile_obj: object to check 
        :param ifacebitmaps: Interface bitmaps for deserialization
        :returns: true if it was filled and volatile, false otherwise
        :type volatile_obj: DataClayObject
        :type ifacebitmaps: dict 
        :rtype: boolean
        """
        
        object_id = volatile_obj.get_object_id()
        if hasattr(self.thread_local_info, "volatiles_under_deserialitzation"): 
            if self.thread_local_info.volatiles_under_deserialitzation is not None: 
                for obj_with_data in  self.thread_local_info.volatiles_under_deserialitzation.values():
                    curr_obj_id = obj_with_data[0]
                    if object_id == curr_obj_id:
                        # deserialize it 
                        metaclass_id = volatile_obj.get_class_extradata().class_id
                        hint = volatile_obj.get_hint()
                        self.get_or_new_volatile_instance_and_load(object_id, metaclass_id, hint,
                                              obj_with_data, ifacebitmaps)
                        return True
        return False

    def add_alias_reference(self, object_id):
        """
        @summary Add +1 reference due to a new alias.
        @param object_id ID of object with alias
        """
        self.alias_references.add(object_id)
        
    def add_session_reference(self, object_id):
        """
        @summary Add +1 reference associated to thread session
        @param object_id ID of object.
        """
        session_id = self.get_session_id()
        if session_id is None:
            # session id can be none in case of when federated
            return
        
        if not object_id in self.references_hold_by_sessions: 
            """ race condition: two objects creating set of sessions at same time """ 
            self.lock(object_id)
            try: 
                if not object_id in self.references_hold_by_sessions: 
                    session_refs = set() 
                    self.references_hold_by_sessions[object_id] = session_refs 
            finally:
                self.unlock(object_id)
        else:
            session_refs = self.references_hold_by_sessions.get(object_id)
        session_refs.add(session_id) 
        
        """ add expiration date of session if not present
        IMPORTANT: if CHECK_SESSION=FALSE then we use a default expiration date for all sessions
        In this case, sessions must be explicitly closed otherwise GC is never going to clean unused objects from sessions.
        Concurrency note: adding two times same expiration date is not a problem since exp. date is the same. We avoid locking.
        """
        if not session_id in self.session_expires_dates:
            if Configuration.CHECK_SESSION:
                """ TODO: implement session control in python """ 
            else:
                expiration_date = Configuration.NOCHECK_SESSION_EXPIRATION
                                    
            """
            // === concurrency note === //
            T1 is here, before put. This is a session that was already used and was restarted.
            T2 is in @getReferenceCounting and wants to remove session since it expired.
            What if T2 removes it after the put?
            Synchronization is needed to avoid this. It is not a big penalty if session expiration date was already added.
            """
            self.lock(session_id)  # Use same locking system for object ids. 
            try: 
                self.session_expires_dates[session_id] = expiration_date
            finally: 
                self.unlock(session_id)
        
    def close_session_in_ee(self, session_id):
        """
        @summary Close session in EE. Subtract session references for GC.
        @param session_id ID of session closing.
        """
        
        logger.debug("[==DGC==] Closing session %s" % str(session_id))

        """ Closing session means set expiration date to now """
        self.session_expires_dates[session_id] = datetime.datetime.now()
    
    def get_retained_references(self):
        """
        @summary Get retained refs by this EE
        @return Retained refs (alias, sessions, ...)
        """
        logger.debug("Getting retained references")
        retained_refs = list() 
        
        """ memory references """
        retained_refs.extend(self.dataclay_heap_manager.get_object_ids_retained())
        logger.debug("Obtained retained references in memory: %s " % str(len(retained_refs)))

        """ session references """ 
        now = datetime.datetime.now()

        sessions_to_close = set()
        for oid in list(self.references_hold_by_sessions.keys()):  # use keys as copy to avoid concurrency problems
            sessions_of_obj = self.references_hold_by_sessions.get(oid)
            logger.debug("[==DGC==] Object %s is retained by sessions %s" % (oid, str(sessions_of_obj)))
            """ create a copy of the list to avoid modification issues while iterating and concurrence problems """
            for cur_session in list(sessions_of_obj):
                
                """ check session expired """
                """
                ==== session counting design - Race condition ==== //
                Race condition: object is send between two nodes and they are both notifying 0 references. This is not
                solved using quarantine in SL since during quarantine period they could do the same and always send 0: while one is
                notifying 0, the other keeps the object, and send to the other before notifying 0.
                In order to avoid this, since session reference is added every time we communicate
                (even between nodes! not only client - node)
                we do NOT remove session reference till GGC asks TWO times
        
                Explicit closes of sessions set expire date to "now" but user can restart a session
                so, even if session is in quarantine, we must check date.
                """
                session_expired = False
                expired_date = self.session_expires_dates.get(cur_session)
                print(expired_date)
                if expired_date is not None and now > expired_date:
                    if cur_session in self.quarantine_sessions:
                        # Session is actually removed 
                        session_expired = True
                        
                        """ check again expiration date to see if it is expired. If expired, remove. """
                        cur_expired_date = self.session_expires_dates.get(cur_session)
                        if cur_expired_date is not None and now > cur_expired_date:
                            """
                            do not remove expiration date from session till there is no objects in that session
                            add it to sessions to close after all processing"""
                            sessions_to_close.add(cur_session)
                            
                    else:
                        self.quarantine_sessions.add(cur_session)
                elif expired_date is not None and now < expired_date:
                    # check if session was in quarantine: if so, remove it from there (session restart)
                    if cur_session in self.quarantine_sessions:
                        self.quarantine_sessions.remove(cur_session)
                        
                if session_expired:
                    """ close session """ 
                    sessions_of_obj.remove(cur_session)
                    
                    """
                    // === concurrency note === //
                    when should we remove an entry in the references_hold_by_sessions map?
                    1 - when no session is using the object
                    2 - when object is not in memory
                    so, we check both here and we remove it if needed:
                    TODO: what if after if, it is added?
                    """
                    logger.debug("Session %s expired" % str(cur_session))
                    if not self.dataclay_heap_manager.exists_in_heap(oid) and len(sessions_of_obj) == 0:
                        logger.debug("Removing session reference for oid %s" % str(oid))
                        for oid, oid_ref in self.references_hold_by_sessions.items():
                            oid_ref.discard(cur_session)  # do not raise key error if not found
                        print(self.references_hold_by_sessions)

                else:
                    retained_refs.append(oid)
        
        """ check closed sessions """ 
        """ Remove all expired sessions if, and only if, there is no object retained by it. 
        TODO: improve this implementation."""
        for session_to_close in sessions_to_close:
            obj_using_session = False
            for sessions_of_obj in self.references_hold_by_sessions.values():
                if session_to_close in sessions_of_obj:
                    obj_using_session = True
                    break
                
            if not obj_using_session:
                del self.session_expires_dates[session_to_close]
        
        logger.debug("Retained references %s" % str(retained_refs))
        return retained_refs

    def get_from_sl(self, object_id):
        """ Get from SL associated to this EE. 
        :param object_id: id of the object to get 
        :type object_id: ObjectID 
        :returns: Bytes of object
        :rtype: Byte array
        """
        return self.ready_clients["@STORAGE"].get_from_db(settings.environment_id, object_id)
    
    def update_to_sl(self, object_id, obj_bytes, dirty):
        """ Update to SL associated to this EE. 
        :param object_id: id of the object  
        :param obj_bytes: bytes to update
        :param dirty: indicates if object is dirty or not
        :returns: None
        :type object_id: ObjectID 
        :type obj_bytes: Byte array 
        :type dirty: Boolean
        :rtype: None
        """
        return self.ready_clients["@STORAGE"].update_to_db(settings.environment_id, object_id, obj_bytes, dirty)




""" Class description goes here. """

"""
Public dataclay functions exported to use (friendly) "from dataclay import ..."
"""
from dataclay.commonruntime.Runtime import getRuntime
from dataclay.commonruntime.Runtime import StorageObject
from dataclay.DataClayObject import DataClayObject
from dataclay.commonruntime.Initializer import *
from dataclay.DataClayObjMethods import dclayMethod
from dataclay.DataClayObjMethods import dclayEmptyMethod





""" Class description goes here. """

from dataclay.util.MgrObject import ManagementObject


class SessionOperation(ManagementObject):
    _fields = ["id",
               "operationID",
               "sessionLocalImplementation",
               "sessionRemoteImplementation",
               ]

    _internal_fields = []
